    body, html {
        font-size: 10px;
        height: 100%;
        width: 100%;
        margin: 0px;
        overflow: hidden;
        float: left;
    }
    
	#smalllogo {
		width:150px;
		height:45px;
	}
	
    #overlay {
    	visibility: hidden;
    	position: absolute;
    	left: 0px;
    	top: 0px;
    	width:100%;
    	height:100%;
    	font-size: 13px;
    	text-align:center;
    	z-index: 1000;
	}
    
    #overlay div {
    	width:325px;
    	margin: 75px auto;
    	background-color: #fff;
    	border:1px solid #000;
    	padding:15px;
    	text-align:center;
	}
	
	#explanation {
		text-align:left;
	}
	
	#source {
		font-size:11px;
		text-align:left;
	}
      
    div.plotWrapper {
        position: relative;
        overflow:hidden;
        height: 100%;
        width:73%;
        float: left;
        margin:0;
    }
      
    #plotToggle {
        z-index: 5;
        width: 73%;
        margin:0;
        overflow:hidden;
        bottom: 0;
        position: absolute;
        float: left;
    }

    div.separator {
        height:100%;
        width:1%;
        float:left;
        cursor:col-resize;
        margin:0;
        background-color:#333333;
    }
    div.separator:hover {
    	background-color:#666666;
    }

    div#menu {
        height: 100%;
        font-family: Helvetica, sans-serif;
        width:26%;
        overflow:hidden;
        float:left;
        padding: 0px;
        margin:0;
    }
    
    div#menutabs {
        background: #eee;
        /*margin-before: 0px !important;*/
        height: 100%;
        margin:0;
    }
    
    .ui-tabs .ui-widget .ui-widget-content .ui-corner-all {
        /*  height: 95%;*/
        padding-left: 0px !important;
        padding-top: 0px;
    }
    div#colorby {
        padding-top: 0px;
        padding-bottom: 0px;
        width:100%;
        height:100%;
        overflow:scroll;
    }
    
    div#keytab {
        height: 94%;
        width:90%;
        padding-top: 0px;
        padding-bottom: 0px;
    }
    
    div#key {
        height: 85%;
        overflow: auto;
    }
    
    table {
        font-family: Helvetica, sans-serif;
        font-size: 12px !important;
    }
    
    select {
        /*  height: 20%;*/
        /*width: 100%;*/
    }
    
    div.colorbox {
        border: 1px solid black;
        height: 18px;
        width: 18px;
    }
    
    div.animationlist {
        margin-top: 1em;
        width: 100%;
        overflow: auto;
        height: 60%;
    }
    
    div.list{
        width: 100%;
        height: 85%;
        overflow: auto;
    }
    
    div.list ul {
        width: 100%;
        padding-left: 0px !important;
        list-style-type: none;
        font-size: 12px !important;
    }
    
    input.button {
        height: 2em;
        width: 100%;
    }
    
    label {
        border:0; 
        font: 9pt Helvetica;
}
    
    label.slidervalue {
        float: right;
        border:0; 
        color:#6cf; 
        font-weight:bold;
        font: 9pt Helvetica;
    }

    .ui-tabs .ui-tabs-nav li a{
        padding: 2px 5px !important;
    }
    
    div#labelby {
        height: 90%;
    }
    
    div#labellist {
        height: 60%;
    }
    
    div.labelsTop {
        height: 30%;
    }
    
    div#labels {
        display: none;
        font-family: courier;
        color: #fff;
        height: 0px;
        float: left;
        position: absolute;
        z-index: 1;
    }
    
    div#taxalabels {
        display: none;
        font-family: courier;
        color: #fff;
        height: 0px;
        float: left;
        position: absolute;
        z-index: 1;
    }
    
    .axislabels {
        color: #fff;
    }
    
    div#labelColorHolder {
        width: 100%;
    }
    
    div#labelColorHolder div {
        float: left;
    }
    
    div#labelColorHolder label{
        float: left;
        width: 50%;
    }
    
    form {
        margin: 0px;
    }
    
    input {
      font-style: 10pt helvetica;
    }
    
    a.mediabutton img {
        background-color: #eee;
    }
    
    a.mediabutton:hover img{
        background-color: #cff;
    }
    
    *.unselectable {
        -moz-user-select: -moz-none;
        -khtml-user-select: none;
        -webkit-user-select: none;

        /*
            Introduced in IE 10.
            See http://ie.microsoft.com/testdrive/HTML5/msUserSelect/
        */
        -ms-user-select: none;
        user-select: none;
    }
    
    .ontop {
        padding: 2px;
        font-style: 9pt helvetica;
        color: #fff;
        border: 1px solid white;
        position: absolute;
    }
    
    .arrow-right {
        opacity: 0;
        position: absolute;
        width: 0;
        height: 0;
        border-top: 5px solid transparent;
        border-bottom: 5px solid transparent;
        border-left: 10px solid white;
    }
    
    .circle {
        position: absolute;
        opacity: 0;
        border: 3px solid white;
        border-radius: 50%;
        width: 10px;
        height: 10px; 
        /* width and height can be anything, as long as they're equal */
    }

    .invisible {
        display: none;
    }
